// ****************************************************************
//
//   IBM Confidential
//   OCO Source Materials
//   <<PART NUMBER - 5724-B75>>
//   (C) Copyright IBM Corp. 1999, 2004 - All Rights Reserved.
//   US Government Users Restricted Rights - Use, duplication or disclosure
//   restricted by GSA ADP Schedule Contract with IBM Corp.
//
// ****************************************************************
package com.ibm.etools.marshall.util;

/**
 * This class defines the behavior for a range of alpha values
 * (ie. the characters 'a'...'z' and 'A'...'Z' in the English language).
 * The Java method Character.isLetter() must return <code>true</code> for
 * a character to be a value in this range.  This class is intended to
 * be used with the possible value construct on a field.
 */

public class CharacterRange implements IRangeObject
{
	private static String copyright = "(c) Copyright IBM Corporation 1999, 2004.";
	static final long serialVersionUID = -4062258957317422887L;

	protected char lowerRange_;
	protected char upperRange_;
/**
 * Creates a new empty instance of an CharacterRange object.
 */

public CharacterRange()
{
	lowerRange_ = 'A';
	upperRange_ = 'A';
}
/**
 * Creates a new instance of an CharacterRange object with the specified upper
 * and lower ranges.
 * @param lowerRange The lower range.
 * @param upperRange The upper range.
 * @exception java.lang.IllegalArgumentException The upper range is less than
 * the lower range, or either the lowerRange or upperRange is not a character.
 */

public CharacterRange(char lowerRange, char upperRange) throws IllegalArgumentException
{
	if (!Character.isLetter(lowerRange))
		throw new IllegalArgumentException(new Character(lowerRange).toString());

	if (!Character.isLetter(upperRange))
		throw new IllegalArgumentException(new Character(upperRange).toString());

	if (upperRange < lowerRange)
		throw new IllegalArgumentException(new Character(upperRange).toString());

	lowerRange_ = lowerRange;
	upperRange_ = upperRange;
}
/**
 * Creates a new object of the same class as this object. It then initializes
 * each of the new object's fields by assigning it the same value as
 * the corresponding field in this object. No constructor is called.
 * @return A new object initialized to the same state as this object.
 */

public Object clone()
{
	try
	{
		CharacterRange newType = (CharacterRange)super.clone();

		return (newType);
	}
	catch (CloneNotSupportedException ex)
	{
		throw new java.lang.InternalError(ex.getMessage());
	}
}
/**
 * This method determines if the two character range objects are equivalent.
 * @return <code>true</code> if this range object is the same as the obj
 * argument; <code>false</code> otherwise.
 * @param obj The reference object with which to compare.
 */

public boolean equals (Object obj)
{
	if (obj == null)
		return (false);

	if (this == obj)
		return (true);

	if (!(obj instanceof CharacterRange))
		return (false);

	CharacterRange tmpRange = (CharacterRange)obj;

	if (lowerRange_ != tmpRange.lowerRange_)
		return (false);

	if (upperRange_ != tmpRange.upperRange_)
		return (false);

	return (true);
}
/**
 * Returns the lower range, or bounds, of the range object as a
 * java.lang.Character object.
 * @return The lower range, or bounds, of the range object.
 */

public Object getLowerRange()
{
	return (new Character(lowerRange_));
}
/**
 * Returns all of the values of the range in a Character array  Each value
 * int the array is a java.lang.Character object.
 * @return All of the values of the range in an object array.
 */

public Object[] getRangeValues()
{
	// Calculate the number of elements for the array

	int arrayLength = 0;

	for (int i = lowerRange_; i <= upperRange_ ; i++)
	{
		if (Character.isLetter ((char)i))
			arrayLength++;
	}

	Character[] values = new Character[arrayLength];

	for (int i = lowerRange_, j = 0; i <= upperRange_ ; i++)
	{
		if (Character.isLetter ((char)i))
		{
			values[j] = new Character((char)i);
			j++;
		}
	}
	
	return (values);
}
/**
 * Returns the upper range, or bounds, of the range object as a
 * java.lang.Character object.
 * @return The upper range, or bounds, of the range object.
 */

public Object getUpperRange()
{
	return (new Character(upperRange_));
}
/**
 * Tests whether the specified value lies within the specifed upper and lower
 * range.
 * @return <code>true</code> if the specified value lies within the upper
 * and lower range; <code>false</code> otherwise.
 * @param lowerRange The lower range limit.
 * @param upperRange The upper range limit.
 * @param value The value to compare against.
 * @exception java.lang.IllegalArgumentException The upper range is less than
 * the lower range, or either the lowerRange, upperRange, or value is not a letter.
 */

public static boolean isInRange (char lowerRange, char upperRange, char value)
	throws IllegalArgumentException
{
	if (!Character.isLetter(lowerRange))
		throw new IllegalArgumentException(new Character(lowerRange).toString());

	if (!Character.isLetter(upperRange))
		throw new IllegalArgumentException(new Character(upperRange).toString());

	if (!Character.isLetter(value))
		throw new IllegalArgumentException(new Character(value).toString());

	if (upperRange < lowerRange)
		throw new IllegalArgumentException(new Character(upperRange).toString());

	if ((value >= lowerRange) && (value <= upperRange))
		return (true);

	return (false);
}
/**
 * Tests whether the specified value lies within the upper and lower range
 * set on the range object.
 * @return <code>true</code> if the specified value lies within the upper
 * and lower range; <code>false</code> otherwise.
 * @param value The value to compare against.  Must be a java.lang.Character object.
 */

public boolean isInRange (Object value)
{
	if (!(value instanceof Character))
		return (false);

	char charVal = ((Character)value).charValue();

	return (isInRange (lowerRange_, upperRange_, charVal));
}
/**
 * Sets the lower range, or bounds, of the range object.
 * @param value The lower range value.    Must be a java.lang.Character object.
 * @exception IllegalArgumentException The value specified is not valid for
 * this range object.
 */

public void setLowerRange (Object value) throws IllegalArgumentException
{
	if (!(value instanceof Character))
		throw new IllegalArgumentException(value.toString());

	lowerRange_ = ((Character)value).charValue();

	return;
}
/**
 * Sets the upper range, or bounds, of the range object.
 * @param value The upper range value.  Must be a java.lang.Character object.
 * @exception IllegalArgumentException The value specified is not valid for
 * this range object.
 */

public void setUpperRange (Object value) throws IllegalArgumentException
{
	if (!(value instanceof Character))
		throw new IllegalArgumentException(value.toString());

	upperRange_ = ((Character)value).charValue();

	return;
}
}
